@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005-2008 Red Hat Inc.
+// Copyright (C) 2005-2009 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -13,7 +13,8 @@
 #include "parse.h"
 #include <string>
 #include <vector>
-#include <iostream>
+//#include <iostream>
+#include <iosfwd>
 #include <sstream>
 #include <map>
 
@@ -22,7 +23,7 @@
 struct derived_probe;
 struct match_node;
 
-struct symresolution_info: public traversing_visitor 
+struct symresolution_info: public traversing_visitor
 {
 protected:
   systemtap_session& session;
@@ -53,6 +54,8 @@ struct typeresolution_info: public visitor
   bool assert_resolvability;
   functiondecl* current_function;
   derived_probe* current_probe;
+  std::vector <const token*> resolved_toks; // account for type mis-
+  std::vector <const token*> printed_toks;  // matches (BZ 9719)
 
   void check_arg_type (exp_type wanted, expression* arg);
   void mismatch (const token* tok, exp_type t1, exp_type t2);
@@ -94,6 +97,7 @@ struct typeresolution_info: public visitor
   void visit_print_format (print_format* e);
   void visit_stat_op (stat_op* e);
   void visit_hist_op (hist_op* e);
+  void visit_cast_op (cast_op* e);
 };
 
 
@@ -114,6 +118,8 @@ struct derived_probe: public probe
   derived_probe (probe* b, probe_point* l);
   probe* base; // the original parsed probe
   virtual probe* basest () { return base->basest(); }
+  // XXX: might be helpful for listing and stepwise expansion, but aliases/wildcards don't show up right
+  // virtual probe* almost_basest () { probe* bb = base->basest(); return (bb == base) ? this : bb; }
   virtual ~derived_probe () {}
   virtual void join_group (systemtap_session& s) = 0;
   virtual probe_point* sole_location () const;
@@ -129,6 +135,10 @@ struct derived_probe: public probe
   // From within unparser::emit_probe, initialized any extra variables
   // in this probe's context locals.
 
+  virtual void emit_probe_local_init (translator_output*) {}
+  // From within unparser::emit_probe, emit any extra processing block
+  // for this probe.
+
 public:
   static void emit_common_header (translator_output* o);
   // from c_unparser::emit_common_header
@@ -173,21 +183,23 @@ struct derived_probe_group
 
 // ------------------------------------------------------------------------
 
+typedef std::map<std::string, literal*> literal_map_t;
+
 struct derived_probe_builder
 {
   virtual void build(systemtap_session & sess,
-		     probe* base, 
+		     probe* base,
 		     probe_point* location,
-		     std::map<std::string, literal*> const & parameters,
+		     literal_map_t const & parameters,
 		     std::vector<derived_probe*> & finished_results) = 0;
   virtual ~derived_probe_builder() {}
   virtual void build_no_more (systemtap_session &) {}
 
-  static bool has_null_param (std::map<std::string, literal*> const & parameters,
+  static bool has_null_param (literal_map_t const & parameters,
                               const std::string& key);
-  static bool get_param (std::map<std::string, literal*> const & parameters,
+  static bool get_param (literal_map_t const & parameters,
                          const std::string& key, std::string& value);
-  static bool get_param (std::map<std::string, literal*> const & parameters,
+  static bool get_param (literal_map_t const & parameters,
                          const std::string& key, int64_t& value);
 };
 
@@ -251,3 +263,5 @@ struct unparser;
 
 
 #endif // ELABORATE_H
+
+/* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
