@@ -102,6 +102,7 @@ static int enable_uprobes(void)
 {
 	int i;
 	char *argv[10];
+	char runtimeko[2048];
 	uid_t uid = getuid();
 	gid_t gid = getgid();
 
@@ -128,14 +129,18 @@ static int enable_uprobes(void)
 	if (run_as(0, 0, 0, argv[0], argv) == 0)
 		return 0;
 
-	dbug(2, "Inserting uprobes module from SystemTap runtime.\n");
+	snprintf (runtimeko, sizeof(runtimeko), "%s/uprobes/uprobes.ko",
+		  (getenv("SYSTEMTAP_RUNTIME") ?: PKGDATADIR "/runtime"));
+	dbug(2, "Inserting uprobes module from SystemTap runtime %s.\n", runtimeko);
 	argv[0] = NULL;
-	return insert_module(PKGDATADIR "/runtime/uprobes/uprobes.ko", NULL, argv);
+
+	return insert_module(runtimeko, NULL, argv);
 }
 
 static int insert_stap_module(void)
 {
 	char bufsize_option[128];
+
 	if (snprintf_chk(bufsize_option, 128, "_stp_bufsize=%d", buffer_size))
 		return -1;
 	return insert_module(modpath, bufsize_option, modoptions);
@@ -187,11 +192,6 @@ static int remove_module(const char *name, int verb)
 	close_ctl_channel();
 
 	dbug(2, "removing module %s\n", name);
-
-	/* Don't remove module when priority is elevated. */
-	if (setpriority(PRIO_PROCESS, 0, 0) < 0)
-		_perr("setpriority");
-
 	ret = delete_module (name, 0);
 	if (ret != 0) {
 		err("Error removing module '%s': %s.\n", name, strerror(errno));
@@ -214,8 +214,17 @@ int init_staprun(void)
 	else if (!attach_mod) {
 		if (need_uprobes && enable_uprobes() != 0)
 			return -1;
-		if (insert_stap_module() < 0)
-			return -1;
+		if (insert_stap_module() < 0) {
+                  /* staprun or stapio might have crashed or been SIGKILL'd,
+                     without first removing the kernel module.  This would block
+                     a subsequent rerun attempt.  So here we gingerly try to
+                     unload it first. */
+                  int ret = delete_module (modname, O_NONBLOCK);
+                  err("Retrying, after attempted removal of module %s (rc %d)\n", modname, ret); 
+                  /* Then we try an insert a second time.  */
+                  if (insert_stap_module() < 0)
+                    return -1;
+                }
                 if (send_relocations() < 0)
                   	return -1;
 	}
@@ -237,6 +246,17 @@ int main(int argc, char **argv)
 		exit(-1);
 	}
 
+	if (getuid() != geteuid()) { /* setuid? */
+		rc = unsetenv("SYSTEMTAP_STAPRUN") ||
+                  unsetenv("SYSTEMTAP_STAPIO") ||
+                  unsetenv("SYSTEMTAP_RUNTIME");
+
+		if (rc) {
+			_perr("unsetenv failed");
+			exit(-1);
+		}
+	}
+
 	setup_signals();
 
 	parse_args(argc, argv);
@@ -278,7 +298,7 @@ int main(int argc, char **argv)
 	if (init_staprun())
 		exit(1);
 
-	argv[0] = PKGLIBDIR "/stapio";
+	argv[0] = getenv ("SYSTEMTAP_STAPIO") ?: PKGLIBDIR "/stapio";
 	if (run_as (1, getuid(), getgid(), argv[0], argv) < 0) {
 		perror(argv[0]);
 		goto err;
@@ -468,4 +488,3 @@ int send_relocations ()
  out:
   return rc;
 }
-
