@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * Symbolic Lookup Functions
- * Copyright (C) 2005-2008 Red Hat Inc.
+ * Copyright (C) 2005-2009 Red Hat Inc.
  * Copyright (C) 2006 Intel Corporation.
  *
  * This file is part of systemtap, and is free software.  You can
@@ -21,11 +21,10 @@
  */
 
 /* XXX: this needs to be address-space-specific. */
-unsigned long _stp_module_relocate(const char *module, const char *section, unsigned long offset)
+static unsigned long _stp_module_relocate(const char *module, const char *section, unsigned long offset)
 {
 	static struct _stp_module *last = NULL;
 	static struct _stp_section *last_sec;
-	unsigned long flags;
 	unsigned i, j;
 
 	/* if module is -1, we invalidate last. _stp_del_module calls this when modules are deleted. */
@@ -114,7 +113,6 @@ static const char *_stp_kallsyms_lookup(unsigned long addr, unsigned long *symbo
 	struct _stp_module *m = NULL;
 	struct _stp_section *sec = NULL;
 	struct _stp_symbol *s = NULL;
-	unsigned long flags;
 	unsigned end, begin = 0;
 
 	m = _stp_mod_sec_lookup(addr, &sec);
@@ -205,14 +203,23 @@ static int _stp_module_check(void)
                           /* XXX: consider using kread() instead of above. */
                           if (theory != practice)
                             {
+                              #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
+                              _stp_error ("%s: inconsistent %s build-id byte #%d "
+                                          "(0x%x [actual] vs. 0x%x [debuginfo])\n",
+                                          THIS_MODULE->name, m->name, j,
+                                          practice, theory);
+                              return 1;
+                              #else
+                              /* This branch is a surrogate for
+                                 kernels affected by Fedora bug
+                                 #465873. */
                               printk(KERN_WARNING
                                      "%s: inconsistent %s build-id byte #%d "
                                      "(0x%x [actual] vs. 0x%x [debuginfo])\n",
                                      THIS_MODULE->name, m->name, j,
                                      practice, theory);
                               break; /* Note just the first mismatch. */
-                              /* XXX: If it were not for Fedora bug #465873,
-                                 we could "return 1;" here to abort the script. */
+                              #endif
                             }
 			} 
 		    }
@@ -227,7 +234,7 @@ static int _stp_module_check(void)
  * a probe because it is too time-consuming. Use at module exit time.
  */
 
-void _stp_symbol_print(unsigned long address)
+static void _stp_symbol_print(unsigned long address)
 {
 	const char *modname;
 	const char *name;
@@ -246,7 +253,7 @@ void _stp_symbol_print(unsigned long address)
 }
 
 /* Like _stp_symbol_print, except only print if the address is a valid function address */
-int _stp_func_print(unsigned long address, int verbose, int exact)
+static int _stp_func_print(unsigned long address, int verbose, int exact)
 {
 	const char *modname;
 	const char *name;
@@ -274,7 +281,7 @@ int _stp_func_print(unsigned long address, int verbose, int exact)
 	return 0;
 }
 
-void _stp_symbol_snprint(char *str, size_t len, unsigned long address)
+static void _stp_symbol_snprint(char *str, size_t len, unsigned long address)
 {
 	const char *modname;
 	const char *name;
