@@ -27,6 +27,7 @@
 #include <linux/init.h>
 #include <linux/debugfs.h>
 #include <linux/relay.h>
+#include <linux/mm.h>
 #include "utt.h"
 
 static int utt_overwrite_flag = 0;
@@ -36,10 +37,10 @@ static int utt_overwrite_flag = 0;
  *
  *	Most of this function is deadcopy of relay_switch_subbuf.
  */
-size_t utt_switch_subbuf(struct utt_trace *utt, struct rchan_buf *buf,
+static size_t utt_switch_subbuf(struct utt_trace *utt, struct rchan_buf *buf,
 			 size_t length)
 {
-	void *old, *new;
+	char *old, *new;
 	size_t old_subbuf, new_subbuf;
 
 	if (unlikely(buf == NULL))
@@ -68,7 +69,7 @@ size_t utt_switch_subbuf(struct utt_trace *utt, struct rchan_buf *buf,
 
 	old = buf->data;
 	new_subbuf = buf->subbufs_produced % buf->chan->n_subbufs;
-	new = buf->start + new_subbuf * buf->chan->subbuf_size;
+	new = (char*)buf->start + new_subbuf * buf->chan->subbuf_size;
 	buf->offset = 0;
 	if (!buf->chan->cb->subbuf_start(buf, new, old, buf->prev_padding)) {
 		buf->offset = buf->chan->subbuf_size + 1;
@@ -121,7 +122,7 @@ static void __utt_timer_init(struct utt_trace * utt)
 	add_timer(&utt->timer);
 }
 
-void utt_set_overwrite(int overwrite)
+static void utt_set_overwrite(int overwrite)
 {
 	utt_overwrite_flag = overwrite;
 }
@@ -129,13 +130,13 @@ void utt_set_overwrite(int overwrite)
 static void utt_remove_root(struct utt_trace *utt)
 {
 	if (utt->utt_tree_root) {
-		if (!_stp_lock_debugfs()) {
+		if (!_stp_lock_transport_dir()) {
 			errk("Unable to lock transport directory.\n");
 			return;
 		}
 		if (simple_empty(utt->utt_tree_root))
 			debugfs_remove(utt->utt_tree_root);
-		_stp_unlock_debugfs();
+		_stp_unlock_transport_dir();
 		utt->utt_tree_root = NULL;
 	}
 }
@@ -169,7 +170,7 @@ err:
 }
 
 
-void utt_trace_cleanup(struct utt_trace *utt)
+static void utt_trace_cleanup(struct utt_trace *utt)
 {
 	if (utt == NULL)
 		return;
@@ -181,7 +182,7 @@ void utt_trace_cleanup(struct utt_trace *utt)
 	_stp_kfree(utt);
 }
 
-int utt_trace_remove(struct utt_trace *utt)
+static int utt_trace_remove(struct utt_trace *utt)
 {
 	if (utt->trace_state == Utt_trace_setup ||
 	    utt->trace_state == Utt_trace_stopped)
@@ -287,11 +288,13 @@ static struct rchan_callbacks utt_relay_callbacks_global = {
 /*
  * Setup everything required to start tracing
  */
-struct utt_trace *utt_trace_setup(struct utt_trace_setup *utts)
+static struct utt_trace *utt_trace_setup(struct utt_trace_setup *utts)
 {
 	struct utt_trace *utt = NULL;
 	struct dentry *dir = NULL;
 	int ret = -EINVAL;
+	u64 npages;
+	struct sysinfo si;
 
 	if (!utts->buf_size || !utts->buf_nr)
 		goto err;
@@ -313,6 +316,25 @@ struct utt_trace *utt_trace_setup(struct utt_trace_setup *utts)
 	if (!utt->dropped_file)
 		goto err;
 
+	npages = utts->buf_size * utts->buf_nr;
+	if (!utts->is_global)
+		npages *= num_possible_cpus();
+	npages >>= PAGE_SHIFT;
+	si_meminfo(&si);
+#define MB(i) (unsigned long)((i) >> (20 - PAGE_SHIFT))
+	if (npages > (si.freeram + si.bufferram)) {
+		errk("Not enough free+buffered memory(%luMB) for log buffer(%luMB)\n",
+		     MB(si.freeram + si.bufferram),
+		     MB(npages));
+		ret = -ENOMEM;
+		goto err;
+	} else if (npages > si.freeram) {
+		/* exceeds freeram, but below freeram+bufferram */
+		printk(KERN_WARNING
+		       "log buffer size exceeds free memory(%luMB)\n",
+		       MB(si.freeram));
+	}
+
 #if (RELAYFS_CHANNEL_VERSION >= 7)
 	if (utts->is_global)
 		utt->rchan = relay_open("trace", dir, utts->buf_size, utts->buf_nr, 
@@ -351,7 +373,7 @@ err:
 	return NULL;
 }
 
-int utt_trace_startstop(struct utt_trace *utt, int start,
+static int utt_trace_startstop(struct utt_trace *utt, int start,
 			unsigned int *trace_seq)
 {
 	int ret;
