@@ -131,7 +131,7 @@ static int check_binary_precision (int precision) {
   return precision;
 }
 
-int _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
+static int _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 {
 	int len;
 	uint64_t num;
@@ -143,8 +143,6 @@ int _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 	int precision;		/* min. # of digits for integers; max
 				   number of chars for from string */
 	int qualifier;		/* 'h', 'l', or 'L' for integer fields */
-	char *write_len_ptr = NULL;
-	int write_len_width = 0;
 
 	/* Reject out-of-range values early */
 	if (unlikely((int) size < 0))
@@ -172,7 +170,7 @@ int _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 		case '#': flags |= STP_SPECIAL; goto repeat;
 		case '0': flags |= STP_ZEROPAD; goto repeat;
 		}
-		
+
 		/* get field width */
 		field_width = -1;
 		if (isdigit(*fmt))
@@ -190,7 +188,7 @@ int _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 		/* get the precision */
 		precision = -1;
 		if (*fmt == '.') {
-			++fmt;	
+			++fmt;
 			if (isdigit(*fmt))
 				precision = skip_atoi(&fmt);
 			else if (*fmt == '*') {
@@ -248,6 +246,11 @@ int _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 					++str;
 				}
 			}
+#ifdef __ia64__
+			if ((str + precision - 1) <= end)
+				memcpy(str, &num, precision); //to prevent unaligned access
+			str += precision;
+#else
 			switch(precision) {
 			case 1:
 				if(str <= end)
@@ -271,14 +274,16 @@ int _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 				str+=8;
 				break;
 			}
+#endif
 			while (len < field_width--) {
 				if (str <= end)
 					*str = '\0';
 				++str;
 			}
 			continue;
-			
+
 		case 's':
+	        case 'M':
 		case 'm':
 			s = va_arg(args, char *);
 			if ((unsigned long)s < PAGE_SIZE)
@@ -298,11 +303,20 @@ int _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 					++str;
 				}
 			}
-			for (i = 0; i < len; ++i) {
-				if (str <= end)
-					*str = *s;
-				++str; ++s;
+			if (*fmt == 'M') {
+			  str = number(str, str + len - 1 < end ? str + len - 1 : end,
+			      (unsigned long) *(uint64_t *) s,
+			      16, field_width, len, flags);
 			}
+			else {
+                                for (i = 0; i < len; ++i) {
+                                        if (str <= end) {
+                                                *str = *s;
+                                        }
+                                        ++str; ++s;
+                                }
+			}
+
 			while (len < field_width--) {
 				if (str <= end)
 					*str = ' ';
@@ -314,7 +328,6 @@ int _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 			       ++str;
 			}
 			continue;
-
 		case 'X':
 			flags |= STP_LARGE;
 		case 'x':
@@ -326,7 +339,7 @@ int _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 			flags |= STP_SIGN;
 		case 'u':
 			break;
-				
+
 		case 'p':
 			/* Note that %p takes an int64_t argument. */
 			len = 2*sizeof(void *) + 2;
@@ -357,16 +370,6 @@ int _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 				     16, field_width, field_width, flags);
 			continue;
 
-		case 'n':
-			write_len_ptr = str;
-			write_len_width = 2;
-			if (field_width == 1)
-				write_len_width = 1;
-			else if (field_width == 4)
-				write_len_width = 4;
-			str += write_len_width;
-			continue;
-			
 		case '%':
 			if (str <= end)
 				*str = '%';
@@ -430,29 +433,6 @@ int _stp_vsnprintf(char *buf, size_t size, const char *fmt, va_list args)
 			     field_width, precision, flags);
 	}
 
-	if (write_len_ptr) {
-		int written;
-		if (likely(str <= end))
-			written = str - write_len_ptr - write_len_width;
-		else
-			written = end - write_len_ptr - write_len_width;
-
-		if (likely(write_len_ptr + write_len_width < end)) {
-			switch (write_len_width) {
-			case 1: 
-				*(uint8_t *)write_len_ptr = (uint8_t)written;
-				break;
-			case 2: 
-				*(uint16_t *)write_len_ptr = (uint16_t)written;
-				break;
-
-			case 4: 
-				*(uint32_t *)write_len_ptr = (uint32_t)written;
-				break;
-			}
-		}
-	}
-	
 	if (likely(str <= end))
 		*str = '\0';
 	else if (size > 0)
